"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_express_1 = require("apollo-server-express");
const ava_1 = __importDefault(require("ava"));
const dataloader_1 = __importDefault(require("dataloader"));
const graphql_request_1 = __importDefault(require("graphql-request"));
const sinon_1 = __importDefault(require("sinon"));
const createGraphqlServer_1 = __importDefault(require("../helpers/createGraphqlServer"));
const createSchemaWithMiddleware_1 = __importDefault(require("../helpers/createSchemaWithMiddleware"));
ava_1.default('uses lazyLoadMap to lazy load the result', async (t) => {
    const schema = createSchemaWithMiddleware_1.default({
        lazyLoadMap: {
            Foo: () => {
                return {
                    id: '1',
                    name: 'foo',
                };
            },
        },
        resolvers: {
            Query: {
                foo: () => {
                    return {
                        id: '1',
                    };
                },
            },
        },
        typeDefs: apollo_server_express_1.gql `
      type Foo {
        id: ID!
        name: String!
      }

      type Query {
        foo: Foo!
      }
    `,
    });
    const graphqlServer = await createGraphqlServer_1.default({
        schema,
    });
    const response = await graphql_request_1.default(graphqlServer.url, apollo_server_express_1.gql `
    {
      foo {
        id
        name
      }
    }
  `);
    t.deepEqual(response, {
        foo: {
            id: '1',
            name: 'foo',
        },
    });
    await graphqlServer.stop();
});
ava_1.default('caches fetched data', async (t) => {
    const lazyLoad = sinon_1.default
        .stub()
        .returns({
        id: '1',
        name: 'foo',
    });
    const schema = createSchemaWithMiddleware_1.default({
        lazyLoadMap: {
            Foo: lazyLoad,
        },
        resolvers: {
            Query: {
                foo: () => {
                    return {
                        id: '1',
                    };
                },
            },
        },
        typeDefs: apollo_server_express_1.gql `
      type Foo {
        id: ID!
        name: String!
      }

      type Query {
        foo: Foo!
      }
    `,
    });
    const graphqlServer = await createGraphqlServer_1.default({
        schema,
    });
    const response = await graphql_request_1.default(graphqlServer.url, apollo_server_express_1.gql `
    {
      foo {
        id
        name
      }
      foo {
        id
        name
      }
    }
  `);
    t.deepEqual(response, {
        foo: {
            id: '1',
            name: 'foo',
        },
    });
    await graphqlServer.stop();
    t.is(lazyLoad.callCount, 1);
});
ava_1.default('respects the original resolver', async (t) => {
    const lazyLoad = sinon_1.default
        .stub()
        .returns({
        id: '1',
        name: 'foo',
    });
    const schema = createSchemaWithMiddleware_1.default({
        lazyLoadMap: {
            Foo: lazyLoad,
        },
        resolvers: {
            Foo: {
                name: (node) => {
                    return node.name.toUpperCase();
                },
            },
            Query: {
                foo: () => {
                    return {
                        id: '1',
                    };
                },
            },
        },
        typeDefs: apollo_server_express_1.gql `
      type Foo {
        id: ID!
        name: String!
      }

      type Query {
        foo: Foo!
      }
    `,
    });
    const graphqlServer = await createGraphqlServer_1.default({
        schema,
    });
    const response = await graphql_request_1.default(graphqlServer.url, apollo_server_express_1.gql `
    {
      foo {
        id
        name
      }
    }
  `);
    t.deepEqual(response, {
        foo: {
            id: '1',
            name: 'FOO',
        },
    });
    await graphqlServer.stop();
});
ava_1.default('does not fetch already available data', async (t) => {
    const lazyLoad = sinon_1.default
        .stub()
        .throws();
    const schema = createSchemaWithMiddleware_1.default({
        lazyLoadMap: {
            Foo: lazyLoad,
        },
        resolvers: {
            Query: {
                foo: () => {
                    return {
                        id: '1',
                        name: 'foo',
                    };
                },
            },
        },
        typeDefs: apollo_server_express_1.gql `
      type Foo {
        id: ID!
        name: String!
      }

      type Query {
        foo: Foo!
      }
    `,
    });
    const graphqlServer = await createGraphqlServer_1.default({
        schema,
    });
    const response = await graphql_request_1.default(graphqlServer.url, apollo_server_express_1.gql `
    {
      foo {
        id
        name
      }
    }
  `);
    t.deepEqual(response, {
        foo: {
            id: '1',
            name: 'foo',
        },
    });
    await graphqlServer.stop();
    t.is(lazyLoad.callCount, 0);
});
ava_1.default('batches multiple requests', async (t) => {
    const lazyLoad = sinon_1.default
        .stub()
        .resolves([
        {
            id: '1',
            name: 'foo',
        },
        {
            id: '2',
            name: 'bar',
        },
    ]);
    const fooLoader = new dataloader_1.default(lazyLoad);
    const schema = createSchemaWithMiddleware_1.default({
        lazyLoadMap: {
            Foo: ({ id }) => {
                return fooLoader.load(id);
            },
        },
        resolvers: {
            Query: {
                foos: () => {
                    return [
                        {
                            id: '1',
                        },
                        {
                            id: '2',
                        },
                    ];
                },
            },
        },
        typeDefs: apollo_server_express_1.gql `
      type Foo {
        id: ID!
        name: String!
      }

      type Query {
        foos: [Foo!]!
      }
    `,
    });
    const graphqlServer = await createGraphqlServer_1.default({
        schema,
    });
    const response = await graphql_request_1.default(graphqlServer.url, apollo_server_express_1.gql `
    {
      foos {
        id
        name
      }
    }
  `);
    t.deepEqual(response, {
        foos: [
            {
                id: '1',
                name: 'foo',
            },
            {
                id: '2',
                name: 'bar',
            },
        ],
    });
    await graphqlServer.stop();
    t.deepEqual(lazyLoad.firstCall.firstArg, [
        '1',
        '2',
    ]);
    t.is(lazyLoad.callCount, 1);
});
